!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA GSFC Land Data Toolkit (LDT) V1.0
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
! !MODULE: GRACEtws_obsMod
! 
! !DESCRIPTION: 
!  This module provides the observation plugin for the 
!  GRACE terrestrial storage observations. 
!   
! !REVISION HISTORY: 
!  14 Mar 2013: Sujay Kumar, Initial Specification
!
module GRACEtws_obsMod
! !USES: 
  use ESMF
  use map_utils

  implicit none

  PRIVATE 
!-----------------------------------------------------------------------------
! !PUBLIC MEMBER FUNCTIONS:
!-----------------------------------------------------------------------------
  PUBLIC :: GRACEtws_obsinit
!-----------------------------------------------------------------------------
! !PUBLIC TYPES:
!-----------------------------------------------------------------------------
  PUBLIC :: GRACEtwsobs
!EOP
  type, public :: gracetwsobsdec
     integer :: nest    
     character*50  :: map_proj
     character*50  :: format
     character*50  :: wstyle
     character*50  :: wopt
     character*100 :: odir

     integer       :: reftime
     integer       :: tdims
     character*100 :: gracefile, gracescalefile
     character*100 :: graceerrfile
     logical       :: startMode
     integer       :: gracenc, gracenr
     real, allocatable :: tvals(:)
     real, allocatable :: time_bounds(:,:)
     real, allocatable :: lwe_thickness(:,:,:)
     real, allocatable :: scalefactor(:,:)  !BZ
     real, allocatable :: merror(:,:)  
     real, allocatable :: lerror(:,:)  
     real, allocatable :: twsavg(:,:)
     real, allocatable :: lisavg(:,:)
     integer, allocatable :: nlisavg(:,:)

     integer            :: b_syr
     integer            :: b_eyr
     
     integer       :: nc,nr
     real          :: datares
     integer, allocatable   :: n111(:)

     integer, allocatable   :: n11(:)
     integer, allocatable   :: n12(:)
     integer, allocatable   :: n21(:)
     integer, allocatable   :: n22(:)

     real, allocatable   :: w11(:)
     real, allocatable   :: w12(:)
     real, allocatable   :: w21(:)
     real, allocatable   :: w22(:)

     integer                :: process_basin_scale
     integer                :: basin_cat_max
     character*100          :: basinmapfile
     real, allocatable      :: basin_cat(:,:)
  end type gracetwsobsdec

  type(gracetwsobsdec) :: GRACEtwsobs

contains
  
!BOP
! 
! !ROUTINE: GRACEtws_obsInit
! \label{GRACEtws_obsInit}
! 
! !INTERFACE: 
  subroutine GRACEtws_obsinit()
! !USES: 
    use ESMF
    use LDT_coreMod
    use LDT_DAobsDataMod
    use LDT_timeMgrMod
    use LDT_logMod

    implicit none
! !ARGUMENTS: 

! 
! !DESCRIPTION: 
!  This subroutine initializes and sets up the data structures required
!  for reading ECV soil moisture data. 
! 
!EOP
    real                    :: run_dd(8)
    integer                 :: rc
    integer                 :: n 
    integer                 :: ftn,c,r
    real                    :: gridDesci(20)

    n = 1
    call ESMF_ConfigGetAttribute(LDT_config,GRACEtwsobs%gracefile, &
         label="GRACE raw data filename:",rc=rc)
    call LDT_verify(rc,'GRACE raw data filename: not defined')

!!!BZ call to read scale factor
    call ESMF_ConfigGetAttribute(LDT_config,GRACEtwsobs%gracescalefile, &
         label="GRACE scale factor filename:",rc=rc)
    call LDT_verify(rc,'GRACE scale factor filename: not defined')

    call ESMF_ConfigGetAttribute(LDT_config,GRACEtwsobs%graceerrfile, &
         label="GRACE measurement error filename:",rc=rc)
    call LDT_verify(rc,'GRACE measurement error filename: not defined')

    call ESMF_ConfigGetAttribute(LDT_config,GRACEtwsobs%odir, &
         label="LIS TWS output directory:",rc=rc)
    call LDT_verify(rc,'LIS TWS output directory: not defined')

    call ESMF_ConfigGetAttribute(LDT_config,GRACEtwsobs%format, &
         label="LIS TWS output format:",rc=rc)
    call LDT_verify(rc,'LIS TWS output format: not defined')

    call ESMF_ConfigGetAttribute(LDT_config,GRACEtwsobs%wopt, &
         label="LIS TWS output methodology:",rc=rc)
    call LDT_verify(rc,'LIS TWS output methodology: not defined')

    call ESMF_ConfigGetAttribute(LDT_config,GRACEtwsobs%wstyle, &
         label="LIS TWS output naming style:",rc=rc)
    call LDT_verify(rc,'LIS TWS output naming style: not defined')

    call ESMF_ConfigGetAttribute(LDT_config,GRACEtwsobs%map_proj, &
         label="LIS TWS output map projection:",rc=rc)
    call LDT_verify(rc,'LIS TWS output map projection: not defined')

    call ESMF_ConfigGetAttribute(LDT_config,GRACEtwsobs%nest, &
         label="LIS TWS output nest index:",rc=rc)
    call LDT_verify(rc,'LIS TWS output nest index: not defined')

    call ESMF_ConfigGetAttribute(LDT_config,GRACEtwsobs%b_syr, &
         label="GRACE baseline starting year:",rc=rc)
    call LDT_verify(rc,'GRACE baseline start year: not defined')

    call ESMF_ConfigGetAttribute(LDT_config,GRACEtwsobs%b_eyr, &
         label="GRACE baseline ending year:",rc=rc)
    call LDT_verify(rc,'GRACE baseline ending year: not defined')

    if(GRACEtwsobs%map_proj.eq."latlon") then 

       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain lower left lat:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(1),rc=rc)
       
       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain lower left lon:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(2),rc=rc)
       
       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain upper right lat:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(3),rc=rc)
       
       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain upper right lon:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(4),rc=rc)
       
       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain resolution (dx):",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(5),rc=rc)
       
       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain resolution (dy):",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(6),rc=rc)       
       
       GRACEtwsobs%datares = min(run_dd(5),run_dd(6))

       GRACEtwsobs%nc    = (nint((run_dd(4)-run_dd(2))/&
            run_dd(5))) + 1
       GRACEtwsobs%nr    = (nint((run_dd(3)-run_dd(1))/&
            run_dd(6))) + 1

       gridDesci = 0 
       gridDesci(1) = 0 
       gridDesci(2) = GRACEtwsobs%nc
       gridDesci(3) = GRACEtwsobs%nr
       gridDesci(4) = run_dd(1)
       gridDesci(5) = run_dd(2)
       gridDesci(6) = 128
       gridDesci(7) = run_dd(3)
       gridDesci(8) = run_dd(4)
       gridDesci(9) = run_dd(5)
       gridDesci(10) = run_dd(6)
       gridDesci(20) = 64

    elseif(GRACEtwsobs%map_proj.eq."lambert") then 
       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain lower left lat:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(1),rc=rc)
       call LDT_verify(rc,'LIS TWS output domain lower left lat: not defined')
       
       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain lower left lon:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(2),rc=rc)
       call LDT_verify(rc,'LIS TWS output domain lower left lon: not defined')
       
       
       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain true lat1:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(3),rc=rc)
       call LDT_verify(rc,'LIS TWS output domain true lat1: not defined')
       
       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain true lat2:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(4),rc=rc)
       call LDT_verify(rc,'LIS TWS output domain true lat2: not defined')
       
       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain standard lon:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(5),rc=rc)
       call LDT_verify(rc,'LIS TWS output domain standard lon: not defined')
       
       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain resolution:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(6),rc=rc)
       call LDT_verify(rc,'LIS TWS output domain resolution: not defined')
       
       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain x-dimension size:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(7),rc=rc)
       call LDT_verify(rc,'LIS TWS output domain x-dimension size: not defined')
       
       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain y-dimension size:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(8),rc=rc)
       call LDT_verify(rc,'LIS TWS output domain y-dimension size: not defined')

    elseif(GRACEtwsobs%map_proj.eq."polar") then 
       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain lower left lat:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(1),rc=rc)
 

       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain lower left lon:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(2),rc=rc)

       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain true lat1:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(3),rc=rc)

       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain true lat2:",rc=rc)       
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(4),rc=rc)

       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain standard lon:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(5),rc=rc)

       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain resolution:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(6),rc=rc)

       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain x-dimension size:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(7),rc=rc)

       call ESMF_ConfigFindLabel(LDT_config,"LIS TWS output domain y-dimension size:",rc=rc)
       call ESMF_ConfigGetAttribute(LDT_config,run_dd(8),rc=rc)

    endif

    GRACEtwsobs%process_basin_scale = 0 

    call ESMF_ConfigGetAttribute(LDT_config,GRACEtwsobs%process_basin_scale, &
         label="GRACE process basin averaged observations:",rc=rc)
    call LDT_verify(rc,&
         'GRACE process basin averaged observations: not defined')

    if(GRACEtwsobs%process_basin_scale.eq.1) then 
       call ESMF_ConfigGetAttribute(LDT_config,GRACEtwsobs%basinmapfile, &
            label="GRACE basin map file:",rc=rc)
       call LDT_verify(rc,&
            'GRACE basin map file: not defined')
!------------------------------------------------------------------------------
! It is assumed that the basin categories are consecutively ordered from 1. 
!------------------------------------------------------------------------------

       allocate(GRACEtwsobs%basin_cat(LDT_rc%lnc(n),LDT_rc%lnr(n)))
       ftn = LDT_getNextUnitNumber()
       open(ftn,file=GRACEtwsobs%basinmapfile, &
            form='unformatted',access='direct',&
            recl=LDT_rc%lnc(n)*LDT_rc%lnr(n)*4)
       read(ftn,rec=1) GRACEtwsobs%basin_cat
       call LDT_releaseUnitNumber(ftn)

       GRACEtwsobs%basin_cat_max = -1

       do r=1,LDT_rc%lnr(n)
          do c=1,LDT_rc%lnc(n)
             if(GRACEtwsobs%basin_cat(c,r).ne.LDT_rc%udef) then 
                if(GRACEtwsobs%basin_cat(c,r).gt.GRACEtwsobs%basin_cat_max) then 
                   GRACEtwsobs%basin_cat_max = nint(GRACEtwsobs%basin_cat(c,r))
                endif
             endif
          enddo
       enddo

    endif

    GRACEtwsobs%startMode = .true. 

    GRACEtwsobs%gracenc = 360
    GRACEtwsobs%gracenr = 180

!  which variable we want in the DA obs computations. 
    call LDT_initializeDAobsEntry(LDT_DAobsData(1)%TWS_obs, "mm",1,1)
    LDT_DAobsData(1)%tws_obs%selectStats = 1

    call LDT_get_julhr(2002,1,1,0,0,0,GRACEtwsobs%reftime)
    
    n =1 

    if(LDT_isLDTatAfinerResolution(n,GRACEtwsobs%datares)) then 

       allocate(GRACEtwsobs%n11(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
       allocate(GRACEtwsobs%n12(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
       allocate(GRACEtwsobs%n21(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
       allocate(GRACEtwsobs%n22(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
       
       allocate(GRACEtwsobs%w11(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
       allocate(GRACEtwsobs%w12(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
       allocate(GRACEtwsobs%w21(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
       allocate(GRACEtwsobs%w22(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
       
       call bilinear_interp_input(n, gridDesci, &
            GRACEtwsobs%n11, &
            GRACEtwsobs%n12, GRACEtwsobs%n21, &
            GRACEtwsobs%n22, GRACEtwsobs%w11, &
            GRACEtwsobs%w12, GRACEtwsobs%w21, &
            GRACEtwsobs%w22)
    else

       allocate(GRACEtwsobs%n11(&
            GRACEtwsobs%nc*&
            GRACEtwsobs%nr))

       call upscaleByAveraging_input(&
            gridDesci,&
            LDT_rc%gridDesc(n,:),&
            GRACEtwsobs%nc*GRACEtwsobs%nr,&
            LDT_rc%lnc(n)*LDT_rc%lnr(n),&
            GRACEtwsobs%n11)
       
    endif

    gridDesci = 0 
    gridDesci(1) = 0
    gridDesci(2) = 360
    gridDesci(3) = 180
    gridDesci(4) = -89.5
    gridDesci(5) = -179.5
    gridDesci(6) = 128
    gridDesci(7) = 89.5
    gridDesci(8) = 179.5
    gridDesci(9) = 1.0
    gridDesci(10) = 1.0
    gridDesci(20) = 64

    allocate(GRACEtwsobs%lisavg(LDT_rc%lnc(n),LDT_rc%lnr(n)))
    allocate(GRACEtwsobs%nlisavg(LDT_rc%lnc(n),LDT_rc%lnr(n)))

    allocate(GRACEtwsobs%n111(LDT_rc%lnc(n)*LDT_rc%lnr(n)))
    call neighbor_interp_input(n, gridDesci,&
         GRACEtwsobs%n111)

    GRACEtwsobs%lisavg = 0 
    GRACEtwsobs%nlisavg = 0 


    call system('mkdir -p '//(LDT_rc%odir))

  end subroutine GRACEtws_obsinit
     
end module GRACEtws_obsMod
